from ellipsis.path.vector.layer.root import add, edit, delete, recover, archive, getBounds, getChanges, getFeaturesByExtent, getFeaturesByIds, listFeatures, editFilter
from ellipsis.path.vector.layer import featureProperty
from ellipsis.path.vector.layer import style
from ellipsis.path.vector.layer import feature
from ellipsis.path.vector.layer import upload
from ellipsis.path.vector.layer import order


